using System.Diagnostics;
using System.Reflection;
using System.Runtime.InteropServices;
using GraphicsManager;
using GraphicsManager.Enums;
using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using GraphicsManager.Objects.Core;
using Luski.GUI.MainScreen.UI;
using Luski.GUI.MainScreen.UI.PublicServers;
using Luski.GUI.StartPage.UI;
using Luski.net;
using Luski.net.Structures.Public;
using OpenTK.Graphics.OpenGL4;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.Desktop;
using Window = GraphicsManager.Window;

namespace Luski.GUI;

public class MainScreenWindow : Window
{
    private static readonly NativeWindowSettings Settings = new()
    {
        Title = "Luski",
        WindowBorder = WindowBorder.Fixed,
        APIVersion = new Version(3, 2),
        API = ContextAPI.OpenGL,
        StartFocused = true,
        Size = new Vector2i((int)(312 * Globals.Settings.Scale), (int)(545 * Globals.Settings.Scale)),
        Icon = Globals.Icon,
        SharedContext = null,
    };

    public TabControl? tc;
    private FlowLayout? channelpicker, friends, friend_request;
    private RoundedButton? FriendManagerBtn;
    Login login;
    CreateAccount ca;
    private static DebugProc DebugMessageDelegate = OnDebugMessage;

    private static void OnDebugMessage(
        DebugSource source, // Source of the debugging message.
        DebugType type, // Type of the debugging message.
        int id, // ID associated with the message.
        DebugSeverity severity, // Severity of the message.
        int length, // Length of the string in pMessage.
        IntPtr pMessage, // Pointer to message string.
        IntPtr pUserParam) // The pointer you gave to OpenGL, explained later.
    {
        // In order to access the string pointed to by pMessage, you can use Marshal
        // class to copy its contents to a C# string without unsafe code. You can
        // also use the new function Marshal.PtrToStringUTF8 since .NET Core 1.1.
        string message = Marshal.PtrToStringAnsi(pMessage, length);

        // The rest of the function is up to you to implement, however a debug output
        // is always useful.

        switch (severity)
        {
            case DebugSeverity.DebugSeverityHigh:
                if ((Globals.Settings.Logs & ConsoleLog.BigErrosForOpenGL) == ConsoleLog.BigErrosForOpenGL)
                {
                    Console.ForegroundColor = ConsoleColor.DarkRed;
                    Console.WriteLine("[{0} source={1} type={2} id={3}] {4}", severity, source, type, id, message);
                    Console.ResetColor();
                }

                break;
            case DebugSeverity.DebugSeverityMedium:
                if ((Globals.Settings.Logs & ConsoleLog.MediumErrosForOpenGL) == ConsoleLog.MediumErrosForOpenGL)
                {
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    Console.WriteLine("[{0} source={1} type={2} id={3}] {4}", severity, source, type, id, message);
                    Console.ResetColor();
                }
                break;
            case DebugSeverity.DebugSeverityLow:
                if ((Globals.Settings.Logs & ConsoleLog.LowErrosForOpenGL) == ConsoleLog.LowErrosForOpenGL)
                {
                    Console.ForegroundColor = ConsoleColor.DarkYellow;
                    Console.WriteLine("[{0} source={1} type={2} id={3}] {4}", severity, source, type, id, message);
                    Console.ResetColor();
                }
                break;
            default:
                if ((Globals.Settings.Logs & ConsoleLog.InfoForOpenGL) == ConsoleLog.InfoForOpenGL)
                {
                    Console.ForegroundColor = ConsoleColor.Green;
                    Console.WriteLine(message);
                    Console.ResetColor();
                }
                break;
        }
    }
    

    public MainScreenWindow() : base(Settings)
    {
        ShowMissingChar = true;
        LogFrames = ((Globals.Settings.Logs & ConsoleLog.DrawFrames) == ConsoleLog.DrawFrames);
        VSync = VSyncMode.On;
        this.TryGetCurrentMonitorScale(out var h, out var v);
        GL.DebugMessageCallback(DebugMessageDelegate, IntPtr.Zero);
        GL.Enable(EnableCap.DebugOutput);
        try
        {
            Globals.DefaultFontFamly = FontFamily.LoadFontFamily(Globals.GetResource("Fonts.OpenSans.zip"), "OpenSans");
            Globals.DefaultFont = FontInteraction.Load(Globals.DefaultFontFamly);
            //Globals.DefaultFont.AddFamily(FontFamily.LoadFontFamily(Globals.GetResource("Fonts.Exo.zip"), "Exo"));
            //Globals.DefaultFont.AddFamily(FontFamily.LoadFontFamily(Globals.GetResource("Fonts.Roboto.zip"), "Roboto"));
            Globals.DefaultFont.AddFamily(FontFamily.LoadFontFamily(Globals.GetResource("Fonts.Noto_Sans_JP.zip"), "Noto_Sans_JP"));
            string t = "";
            for (int i = 9199; i < 9205; i++)
            {
                t += (char)i;
            }
            //Console.WriteLine(t);
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }
        
        
        
        Globals.DefaultFont.PixelHeight = (uint)(20 * Globals.Settings.Scale);
        Globals.DefaultFont.FontSize = FontSize.Regular;
        Globals.TopTimeFont = Globals.DefaultFont.Clone();
        Globals.TopTimeFont.PixelHeight = (uint)(12 * Globals.Settings.Scale);
        Globals.TopTimeFont.FontSize = FontSize.Regular;
        Globals.MessageFont = Globals.DefaultFont.Clone();
        Globals.MessageFont.PixelHeight = (uint)(17 * Globals.Settings.Scale);
        Globals.MessageFont.ExtraLinePixels = (uint)(5 * Globals.Settings.Scale);
        Globals.MessageFont.FontSize = FontSize.Regular;
        Globals.SmallTimeFont = Globals.DefaultFont.Clone();
        Globals.LuskiTexture = TextureManager.GetTextureResource("Luski.png");
        try
        {
            CenterWindow(Globals.Settings.Display);
        }
        catch (Exception e)
        {
            CenterWindow();
        }
        WindowLoaded += OnWindowLoaded;
    }

    private async Task OnWindowLoaded(Window arg)
    {
        string r = new HttpClient()
            .GetAsync(
                $"https://www.jacobtech.com/Updater/GetProgramVersion?directory=Luski&branch={Globals.UpdaterSettings.Branch.ToString()}&selfcontained={Globals.UpdaterSettings.SelfContained.ToString().ToLower()}&platform={Globals.UpdaterSettings.Platform}")
            .Result.Content.ReadAsStringAsync().Result;
        if (Globals.UpdaterSettings.AutoUpdateCheck && r !=
            FileVersionInfo.GetVersionInfo(Assembly.GetExecutingAssembly().Location).FileVersion)
        {
            var update = new UpdateWindow();
            var result = update.ShowDialogue(this);
            if (result == UpdateWindow.DialogueResult.Yes)
            {
                Globals.Download = true;
                Close();
            }
        }
        await LoginOnChangeToApp();
    }
    
    public FlowLayout ser;
    private UserControl? SerBox;
    public PublicChat pc;

    public async Task LoadPublicServer(PublicServer? Server)
    {
        GL.Enable(EnableCap.DepthTest);
        GL.DepthFunc(DepthFunction.Always);
        if (Server is null) return;
        if (!Server.IsLogedIn)
        {
            ServerLoginOverlay SLO = new(Server.Domain);
            Controls.Add(SLO);
            return;
        }
        BlockDraw = true;
        try
        {
            #region Box Init

            if (SerBox is null)
            {
                SerBox = new()
                {
                    Location = new(ser.Size.X, 0, 0),
                    Size = new(Size.X - ser.Size.X, Size.Y),
                    Anchor = ObjectAnchor.All,
                    BackgroundColor = new(20, 20, 20, 255)
                };
                Controls.Add(SerBox);
                SerBox.LoadToParent(this, this);
            }

            SerBox.Controls.Clear();

            #endregion
            
            #region Channel Selector Init

            SocketChannel current_channel = await Server.User.GetSelectedChannel(CancellationToken.None);
            List<SocketCategory> parents = new();
            SocketCategory? cur = await current_channel.GetParent();
            while (cur is not null)
            {
                parents.Add(cur);
                cur = await cur.GetParent();
            }

            parents.Reverse();
            ChannelSelector cs = await ChannelSelector.MakeSelector(parents[0]);
            cs.BackgroundColor = new(34, 34, 34, 255);
            cs.Size = new((int)(307 * Globals.Settings.Scale), SerBox.Size.Y - (int)(54 * Globals.Settings.Scale));
            cs.Anchor = ObjectAnchor.Top | ObjectAnchor.Left | ObjectAnchor.Bottom;
            
            parents.RemoveAt(0);
            SerBox.Controls.Add(cs);
            cs.ForceDistanceUpdate(SerBox);

            #endregion

            #region Chat Init

            pc = new()
            {
                Anchor = ObjectAnchor.All,
                Location = new(cs.Size.X, 0, 0),
                Size = new(SerBox.Size.X - cs.Size.X, SerBox.Size.Y),
            };
            SerBox.Controls.Add(pc);
            pc.LoadToParent(SerBox, this);
            pc.ForceDistanceUpdate();
            pc.MessageFlow.ForceDistanceUpdate(pc);
            #endregion

            #region Channel Selector Loader

            _ = cs.Load(current_channel, parents);

            #endregion

            #region User Icon
            Role[] ra = await Server.User.GetRoles();
            Color c = ra[0].Color;
            Color4 c4 = new(c.R, c.G, c.B, c.A);
            IRenderObject u = await Server.User.MakeRct(new((int)(46 * Globals.Settings.Scale)), false);
            int ii = (int)(4 * Globals.Settings.Scale);
            u.Location = new(ii, cs.Size.Y + ii, 0);
            u.Anchor = ObjectAnchor.Bottom | ObjectAnchor.Left;
            SerBox.Controls.Add(u);
            u.LoadToParent(SerBox, this);
            u.ForceDistanceUpdate();
            Label ul = new Label(Globals.DefaultFont)
            {
                Anchor = u.Anchor,
                Text = Server.User.DisplayName,
                Color = c4
            };

            ul.Location = new(u.Location.X + u.Size.X + (int)(5 * Globals.Settings.Scale),
                (u.Location.Y + (u.Size.Y / 2) - (ul.PostiveTrueHeight / 2) - ul.Size.Y + ul.TrueHeight), 0);
            SerBox.Controls.Add(ul);
            Rectangle setting = new(TextureManager.GetTextureResource("settings.png"))
            {
                Location = new(cs.Size.X - (int)(40 * Globals.Settings.Scale), cs.Size.Y + (int)(Globals.Settings.Scale * 11),0),
                Size = new((int)(32 * Globals.Settings.Scale)),
                Shader = Rectangle.DefaultAlphaShader[Context],
                BackgroundColor = Color4.Gray,
                Anchor = ObjectAnchor.Bottom | ObjectAnchor.Left
            };
            setting.MouseEnter += o =>
            {
                setting.BackgroundColor = Color4.White;
                return Task.CompletedTask;
            };
            setting.MouseLeave += o =>
            {
                setting.BackgroundColor = Color4.Gray;
                return Task.CompletedTask;
            };
            setting.Clicked += SettingOnClicked;
            setting.ForceDistanceUpdate(SerBox);
            SerBox.Controls.Add(setting);
            ForceUpdate(new (Size));
            TryDraw();
            #endregion
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }

        BlockDraw = false;
    }

    private Task SettingOnClicked(IRenderObject arg)
    {
        SettingsMenu sm = new();
        Controls.Add(sm);
        Globals.ms.DrawFrame();
        return Task.CompletedTask;
    }

    public async Task LoadMainServer(MainServer Server)
    {
        if (SerBox is null)
            SerBox = new()
            {
                Location = new(ser.Size.X, 0, 0),
                Size = new(Size.X - ser.Size.X, Size.Y),
                Anchor = ObjectAnchor.All
            };
        Controls.Add(SerBox);
        SerBox.ForceDistanceUpdate();
        SerBox.Controls.Clear();
    }

    private async Task<Task> LoginOnChangeToApp()
    {
        Controls.Clear();
        BlockDraw = true;
        Title = "Luski";
        Size = new((int)(1332 * Globals.Settings.Scale), (int)(866 * Globals.Settings.Scale));
        try
        {
            CenterWindow(Globals.Settings.Display);
        }
        catch (Exception e)
        {
            CenterWindow();
        }
        WindowBorder = WindowBorder.Resizable;
        BackgroundColor = new Color4(20, 20, 20, 255);

        Controls.Add(ser = new FlowLayout()
        {
            BackgroundColor = new(26, 26, 26, 255),
            Size = new((int)(68 * Globals.Settings.Scale), (int)(868 * Globals.Settings.Scale)),
            Anchor = ObjectAnchor.Top | ObjectAnchor.Left | ObjectAnchor.Bottom,
            Location = new(0,0,0)
        });
        DrawFrame();
        DateTime utcNow = DateTime.UtcNow;
        Task.WhenAll(Globals.ServersLoading.ToArray()).Wait();
        foreach (PublicServer pser in Globals.Luski.LoadedServers)
        {
            ServerIcon<PublicServer> si = new ServerIcon<PublicServer>(pser);
            ser.Controls.Add(si);
        }

        AddServerIcon asi = new();
        asi.Clicked += AddButtonClicked;
        ser.Controls.Add(asi);


        
        
        if (ser.Controls.Length > 1) (ser.Controls[0] as ServerIcon<PublicServer>)!.LoadServer().Start();
        DrawFrame();
        MainShow += OnMainShow;
        return Task.CompletedTask;
    }
    
    private Task AddButtonClicked(IRenderObject arg)
    {
        AddServerOverlay aso = new();
        aso.Clicked += AsoOnClicked;
        Controls.Add(aso);
        
        TryDraw();
        OnResize(new(Size));
        return Task.CompletedTask;
        
    }

    private Task AsoOnClicked(IRenderObject arg)
    {
        Globals.ms.Controls.Remove(arg);
        Globals.ms.DrawFrame();
        AddServerOverlay aso = (arg as AddServerOverlay)!;
        aso.Clicked -= AsoOnClicked;
        aso.Clean();
        aso = null!;
        return Task.CompletedTask;
    }

    private Task OnMainShow()
    {
        if (Globals.Luski.MainServer is not null && Globals.Luski.MainServer.IsLogedIn)
        {
            Globals.Luski.MainServer.OnError += LuskiOnOnError;
        }

        return Task.CompletedTask;
    }

    public event Func<Task>? MainShow;

    private Task LuskiOnOnError(Exception arg)
    {
        Console.WriteLine(arg);
        return Task.CompletedTask;
    }

}